@@ -17,8 +17,11 @@
 package main
 
 import (
+	"bytes"
 	"flag"
 	"fmt"
+	"go/parser"
+	"go/token"
 	"io"
 	"io/fs"
 	"io/ioutil"
@@ -26,19 +29,16 @@ import (
 	"path/filepath"
 	"strings"
 
+	goformat "go/format"
+
+	"github.com/goplus/gop"
 	"github.com/goplus/gop/format"
 )
 
 var (
 	procCnt    = 0
 	walkSubDir = false
-	extGops    = map[string]struct{}{
-		".go":  {},
-		".gop": {},
-		".spx": {},
-		".gmx": {},
-	}
-	rootDir = ""
+	rootDir    = ""
 )
 
 var (
@@ -56,7 +56,7 @@ func report(err error) {
 	os.Exit(2)
 }
 
-func processFile(filename string, in io.Reader, out io.Writer) error {
+func processFile(filename string, class bool, in io.Reader, out io.Writer) error {
 	if in == nil {
 		var err error
 		in, err = os.Open(filename)
@@ -68,8 +68,22 @@ func processFile(filename string, in io.Reader, out io.Writer) error {
 	if err != nil {
 		return err
 	}
-
-	res, err := format.Source(src, filename)
+	var res []byte
+	if filepath.Ext(filename) == ".go" {
+		fset := token.NewFileSet()
+		f, err := parser.ParseFile(fset, filename, src, parser.ParseComments)
+		if err != nil {
+			return err
+		}
+		var buf bytes.Buffer
+		err = goformat.Node(&buf, fset, f)
+		if err != nil {
+			return err
+		}
+		res = buf.Bytes()
+	} else {
+		res, err = format.Source(src, class, filename)
+	}
 	if err != nil {
 		return err
 	}
@@ -99,7 +113,15 @@ func processFile(filename string, in io.Reader, out io.Writer) error {
 	return err
 }
 
-func walk(path string, d fs.DirEntry, err error) error {
+type walker struct {
+	dirMap map[string]func(ext string) (ok bool, class bool)
+}
+
+func newWalker() *walker {
+	return &walker{dirMap: make(map[string]func(ext string) (ok bool, class bool))}
+}
+
+func (w *walker) walk(path string, d fs.DirEntry, err error) error {
 	if err != nil {
 		fmt.Fprintln(os.Stderr, err)
 	} else if d.IsDir() {
@@ -108,10 +130,41 @@ func walk(path string, d fs.DirEntry, err error) error {
 		}
 	} else {
 		// Directories are walked, ignoring non-Gop files.
+		dir, _ := filepath.Split(path)
+		fn, ok := w.dirMap[dir]
+		if !ok {
+			if mod, err := gop.LoadMod(path, nil, &gop.Config{DontUpdateGoMod: true}); err == nil {
+				fn = func(ext string) (ok bool, class bool) {
+					switch ext {
+					case ".go", ".gop":
+						ok = true
+					case ".gopx", ".spx", ".gmx":
+						ok, class = true, true
+					default:
+						_, class = mod.IsClass(ext)
+						if class {
+							ok = true
+						}
+					}
+					return
+				}
+			} else {
+				fn = func(ext string) (ok bool, class bool) {
+					switch ext {
+					case ".go", ".gop":
+						ok = true
+					case ".gopx", ".spx", ".gmx":
+						ok, class = true, true
+					}
+					return
+				}
+			}
+			w.dirMap[dir] = fn
+		}
 		ext := filepath.Ext(path)
-		if _, ok := extGops[ext]; ok {
+		if ok, class := fn(ext); ok {
 			procCnt++
-			if err = processFile(path, nil, os.Stdout); err != nil {
+			if err = processFile(path, class, nil, os.Stdout); err != nil {
 				report(err)
 			}
 		}
@@ -129,12 +182,12 @@ func main() {
 			report(fmt.Errorf("error: cannot use -w with standard input"))
 			return
 		}
-		if err := processFile("<standard input>", os.Stdin, os.Stdout); err != nil {
+		if err := processFile("<standard input>", false, os.Stdin, os.Stdout); err != nil {
 			report(err)
 		}
 		return
 	}
-
+	walker := newWalker()
 	for _, path := range args {
 		walkSubDir = strings.HasSuffix(path, "/...")
 		if walkSubDir {
@@ -142,7 +195,7 @@ func main() {
 		}
 		procCnt = 0
 		rootDir = path
-		filepath.WalkDir(path, walk)
+		filepath.WalkDir(path, walker.walk)
 		if procCnt == 0 {
 			fmt.Println("no Go+ files in", path)
 		}
