@@ -121,7 +121,7 @@ func testFrom(t *testing.T, fpath, sel string, mode int) {
 
 	if (mode & excludeFormatSource) == 0 {
 		t.Run("format.Source "+fpath, func(t *testing.T) {
-			res, err := format.Source(src, fpath)
+			res, err := format.Source(src, false, fpath)
 			if err != nil {
 				t.Fatal("Source failed:", err)
 			}
